"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const tmp = __importStar(require("tmp"));
const generate_1 = require("../flamegraph/generate");
const processData_1 = require("../analysis/processData");
const profiles = require('../fixtures/profiles.json');
const snapshotsDir = path_1.default.join(__dirname, '../fixtures/snapshots');
// TODO: these are black box tests for now but should be refactored to be unit tests
// TODO: modules that output files should be modified not to and wrapped by a centralized file output helper
// TODO: consider also making file output / github / CI integration another package within this repo
describe('flamegrill', () => {
    it('generates expected output', () => __awaiter(this, void 0, void 0, function* () {
        jest.setTimeout(30000);
        expect.assertions(89);
        const outdir = tmp.dirSync({ unsafeCleanup: true });
        yield Promise.all(Object.keys(profiles).map(key => {
            let logfile = require.resolve(path_1.default.join('../fixtures', profiles[key].logFile));
            let outfile = path_1.default.join(outdir.name, key);
            return generate_1.generateFlamegraph(logfile, outfile);
        }));
        yield Promise.all(Object.keys(profiles).map(key => {
            let logfile = require.resolve(path_1.default.join('../fixtures', profiles[key].reference.logFile));
            let outfile = path_1.default.join(outdir.name, key + '_ref');
            return generate_1.generateFlamegraph(logfile, outfile);
        }));
        Object.keys(profiles).forEach(key => {
            // TODO: this code block is duplicating code in flamegrill.ts and should be removed as code is refactored.
            let datafileBefore = path_1.default.join(outdir.name, key + '_ref.data.js');
            let datafileAfter = path_1.default.join(outdir.name, key + '.data.js');
            let regressionfile = path_1.default.join(outdir.name, key + '.regression.txt');
            const analysis = processData_1.checkForRegressions(datafileBefore, datafileAfter);
            if (analysis.isRegression) {
                fs_1.default.writeFileSync(regressionfile, analysis.summary);
            }
        });
        const snapshotFiles = fs_1.default.readdirSync(snapshotsDir);
        const testFiles = fs_1.default.readdirSync(outdir.name);
        expect(testFiles).toEqual(snapshotFiles);
        testFiles.forEach(file => {
            // Some generated output creates files with \r\n. Some environments spit out \n.
            // Ignore line break types when comparing results.
            const analysis = fs_1.default.readFileSync(path_1.default.join(snapshotsDir, file), 'utf8').split(/\r?\n/g);
            const output = fs_1.default.readFileSync(path_1.default.join(outdir.name, file), 'utf8').split(/\r?\n/g);
            expect(output).toEqual(analysis);
        });
        outdir.removeCallback();
    }));
});
